#!/usr/bin/perl -w

#-------------------------------------------
# Copyright (c) Craig D. Jensen 2004-2007
# Genweb Administration panel
#-------------------------------------------
package	esmith::FormMagick::Panel::geneweb;

use strict;
#use warnings;
use esmith::FormMagick;
use esmith::ConfigDB;
use esmith::NetworksDB;
use esmith::util;
use File::Basename;
use Exporter;
use Net::IPv4Addr;

our @ISA = qw(esmith::FormMagick Exporter);

our @EXPORT = qw(
    get_dbdir
    must_exist
    get_prop
    get_value
    get_only
    change_settings
    must_be_local
);

our $VERSION = sprintf '%d.%03d', q$Revision: 0.09 $ =~ /: (\d+).(\d+)/;

our $db = esmith::ConfigDB->open || die "Couldn't open ConfigDB\n";
our $networks = esmith::NetworksDB->open || die "Couldn't open NetworksDB\n";

=pod

=head1 NAME

esmith::FormMagick::Panels::geneweb - Panel providing exclusive single
IP assignment for it's administrative access (:2316)

=head1 SYNOPSYS

    use esmith::FormMagick::Panel::geneweb;

    my $panel = esmith::FormMagick::Panel::geneweb->new;
    $panel->display();

=head1 DESCRIPTION

Geneweb geneology server installs two separate server accesses, the database
interface (accessed at $domain:2317/DBname); the administration interface 
(accessed at $domain:2316 but only by the single IP assigend by the file 
/opt/geneweb/gw/gw/only.txt which defaults to 127.0.0.1). This panel provides a
'non-commandline' form to enter an alternative IP on the LAN from which to 
administer Geneweb.

=cut

# {{{ new

=head2 new()

Exactly as for esmith::FormMagick

=begin testing

$ENV{ESMITH_CONFIG_DB} = "10e-smith-base/configuration.conf"
$ENV{ESMITH_NETWORKS_DB} = "10e-smith-base/networks.conf"

use_ok('esmith::FormMgick::Panel::geneweb');
use vars qw($panel);
ok($panel = esmith::FormMagick::Panel::geneweb->new(), "Create panel object");
isa_ok($panel, 'esmith::FormMagick::Panel::geneweb');

=end testing

=cut



sub new
{
        shift;
        my $self = esmith::FormMagick->new();
        $self->{calling_package} = (caller)[0];
        bless $self;
	#$self->debug(1);
	return $self;
}

=head2 get_prop

=cut

sub get_prop
{
	my $fm = shift if (ref($_[0]) );
	my $item = shift;
	my $prop = shift;
	
	my $record = $db->get($item);
	if ($record) {
	    return $record->prop($prop);
	}
	else {
	    return '';
	}
}

=head2 get_value

=cut

sub get_value
{
	my $fm = shift;
	my $item = shift;
	my $record = $db->get($item);
	#return ($db->get("geneweb")->prop($item));
	if ($record) {
	    return $record->value();
	}
	else {
	    return '';
	}
}
    
=head2 get_only

method to retrieve the value of /opt/geneweb/gw/gw/only.txt for the form
=cut

sub get_only
{
        return $db->get_prop("geneweb", "only");
}
    
=head2 get_dbdir

method to retrieve the value of /opt/geneweb/bases or other location for the form
=cut

sub get_dbdir
{
        return $db->get_prop("geneweb", "DBDir");
}

=head2 change_settings

Sub to change settings as per user input in the panel
=cut

sub change_settings
{
	my $self = shift;
	my $q = $self->{'cgi'};
	my $n_gw_ip = $q->param("geneweb_only") || '';
	my $n_gw_dbdir = $q->param("geneweb_dbdir") || '';
	if (!($n_gw_ip || $n_gw_dbdir)) {
	    return $self->success("NO_CHANGE")
	}
	my $gw_ip = get_only();
	my $gw_dbdir = get_dbdir();
	if (($n_gw_ip eq $gw_ip) && ($n_gw_dbdir eq $gw_dbdir)) {
	    return $self->success("NO_CHANGE")
	}
	if ($n_gw_ip) { $db->set_prop("geneweb", "only", $n_gw_ip) }
	if ($n_gw_dbdir) { $db->set_prop("geneweb", "DBDir", $n_gw_dbdir) }

        unless ( system ( "/sbin/e-smith/signal-event", "geneweb-modify" ) == 0 )
	{
	    $self->error("ERROR_UPDATING");
	    return undef;
        }
	return $self->success("SUCCESS");
}


=head2 valid_ip_number

subroutine to validate IP input.
=cut

sub must_be_local
{
    my $self = shift;
    my $q = $self->{cgi};
    my $gw_ip = $q->param("geneweb_only" || '');
    if (!($gw_ip)) { return 'OK'}
    if (!( $gw_ip =~ /^((([01]?[0-9]{1,2}|[2][0-5]{1,2}).){3}([01]?[0-9]{1,2}|[2][0-5]{1,2}))$/ )) 
    { 
	return 'NOT_VALID_IP';
    }
    # Make sure that the IP is indeed local.
    my $ndb = esmith::NetworksDB->open_ro;
    my @local_list = $ndb->local_access_spec;

    foreach my $spec (@local_list)
    {
        if (Net::IPv4Addr::ipv4_in_network($spec, $gw_ip))
        {
	    return 'OK';
    	}
    }
    # Not OK. The IP is not on any of our local networks.
    return 'ERR_IP_NOT_LOCAL';
}

 
=head2 valid_base_directory

subroutine to validate base directory.
=cut

sub must_exist
{
    my $self = shift;
    my $q = $self->{cgi};
    my $gw_dbdir = $q->param("geneweb_dbdir") || '';
    if ($gw_dbdir) {
	if (! -d "$gw_dbdir")  { return 'ERR_DIR_NOT_EXIST' } 
    #    system("su geneweb -c 'touch $gw_dbdir/toto'") == 0
    #    	or return 'DIR_NOT_WRITEABLE'; 
    #    system("su geneweb -c 'rm -f $gw_dbdir/toto'")
    #    return 'DIR_EXIST_AND_WRITEABLE'; 
    }
    return 'OK';
}

1;
